import 'dart:io';
import 'dart:typed_data';
import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:flutter/material.dart';
import 'package:flutter_local_notifications/flutter_local_notifications.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:flutter_svg/svg.dart';
import 'package:get/get.dart';
import 'notification/firebase_cloud_messaging.dart';
import 'notification/local_notification_setting.dart';
import 'provider/user_state.dart';
import 'screen/alim/alim_main_page.dart';
import 'screen/camera/camera_main.dart';
import 'screen/monitoring/monitoring_main_screen.dart';
import 'screen/setting/setting_main_screen.dart';
import 'screen/util/font/font.dart';

class BottomNavigator extends StatefulWidget {
  static final String id = '/bottom';

  const BottomNavigator({Key? key}) : super(key: key);

  @override
  State<BottomNavigator> createState() => _BottomNavigatorState();
}

class _BottomNavigatorState extends State<BottomNavigator> with TickerProviderStateMixin {
  ///fcm
  final FlutterLocalNotificationsPlugin flutterLocalNotificationsPlugin = FlutterLocalNotificationsPlugin();
  AndroidNotificationChannel? androidNotificationChannel;

  final storage = new FlutterSecureStorage();
  final us = Get.put(UserState());
  List<Widget> _widgetOptions = [];
  late TabController _bottomTabController;
  int _currentIndex = 0;
  String? fontSizes;
  bool _isLoading = true;
  @override
  void initState() {
    Future.delayed(Duration.zero,()async{
      print('바뀐 비빌번호 : ${us.userList[0]['pw']}');
      fontSizes = (await storage.read(key: "fontSizes"));
      switch (fontSizes) {
        case '2' :
          us.userFont.value = 2;
          break;
        case '1' :
          us.userFont.value = 1;
          break;
        case '0' :
          us.userFont.value = 0;
          break;
      }
      print('지정해둔 폰트는 ?? ${us.userFont.value}');
      _isLoading = false;
      setState(() {});
    });
    /// Fcm
    _requestPermissions();
    LocalNotifyCation().initializeNotification();
    FCM().setNotifications();
    // Future.delayed(Duration.zero,()async{
    //   await FirebaseMessaging.instance.setForegroundNotificationPresentationOptions(
    //     alert: true,
    //     badge: true,
    //     sound: true,
    //   );
    // });
    super.initState();
    _widgetOptions = [MonitoringMainPage(),CameraMain(), AlimScreen(),SettingMain()];
    // _widgetOptions = [MonitoringTestPage(),CameraMain(), AlimScreen(),SettingMain()];
    _bottomTabController = TabController(length: 4, vsync: this);
    // _bottomTabController.animateTo(0);
  }

  @override
  Widget build(BuildContext context) {
    return _isLoading?Container():Scaffold(
      bottomNavigationBar: Container(
        padding: Platform.isAndroid
            ? const EdgeInsets.all(0)
            : const EdgeInsets.only(bottom: 10),
        child: TabBar(
          onTap: (index) {
            setState(() {
              _currentIndex = index;
            });
          },
          dividerColor: Colors.transparent,
          indicatorColor: Colors.transparent,
          indicatorSize: TabBarIndicatorSize.label,
          controller: _bottomTabController,
          unselectedLabelStyle: hintf14w700,
          labelStyle: f14w700,
          labelColor: Colors.black,
          padding: EdgeInsets.only(left: 10,right: 10,bottom: 8),
          labelPadding: EdgeInsets.zero,
          tabs: <Widget>[
            Tab(
              icon:SvgPicture.asset(
                'assets/icon/monitoring.svg',
                width: 24,
                height: 24,
                colorFilter:_currentIndex == 0
                    ?ColorFilter.mode(Colors.black, BlendMode.srcIn)
                    :ColorFilter.mode(Colors.grey, BlendMode.srcIn),
              ),
              text: '홈',
            ),
            Tab(
              icon:SvgPicture.asset(
                'assets/icon/camera.svg',
                width: 24,
                height: 24,
                colorFilter:_currentIndex == 1
                    ?ColorFilter.mode(Colors.black, BlendMode.srcIn)
                    :ColorFilter.mode(Colors.grey, BlendMode.srcIn),
              ),
              text: '카메라',
            ),
            Tab(
              icon:SvgPicture.asset(
                'assets/icon/notification.svg',
                width: 24,
                height: 24,
                colorFilter:_currentIndex == 2
                    ?ColorFilter.mode(Colors.black, BlendMode.srcIn)
                    :ColorFilter.mode(Colors.grey, BlendMode.srcIn),
              ),
              text: '알림',
            ),
            Tab(
              icon:SvgPicture.asset(
                'assets/icon/setting.svg',
                width: 24,
                height: 24,
                colorFilter:_currentIndex == 3
                    ?ColorFilter.mode(Colors.black, BlendMode.srcIn)
                    :ColorFilter.mode(Colors.grey, BlendMode.srcIn),
              ),
              text: '설정',
            ),
          ],
        ),
      ),
      body: TabBarView(
        physics: const NeverScrollableScrollPhysics(),
        children: _widgetOptions,
        controller: _bottomTabController,
      ),
    );
  }
  /// 알림 권한 여부 설정
  void _requestPermissions() {
     Int64List vibrationPatterns = Int64List(4);
    vibrationPatterns[0] = 0;      // 진동 시작 전 대기 시간 (0초)
    vibrationPatterns[1] = 1000;   // 첫 번째 진동 시간 (1초)
    vibrationPatterns[2] = 1000;   // 진동과 진동 사이 대기 시간 (1초)
    vibrationPatterns[3] = 2000;   // 두 번째 진동 시간 (2초)
    vibrationPatterns[3] = 2000;
    // 두 번째 진동 시간 (2초)
    /// 안드로이드 일때
    if (Platform.isAndroid) {
      FirebaseMessaging.instance.requestPermission(
        badge: true,
        alert: true,
        sound: true,

      );
      var channel = const AndroidNotificationChannel(
        'sSolution', 'sSolution2',
        description: 'this is sSolution channel', // description
        importance: Importance.high,
        enableVibration: true,
      );
      flutterLocalNotificationsPlugin
          .resolvePlatformSpecificImplementation<AndroidFlutterLocalNotificationsPlugin>()
          ?.createNotificationChannel(channel);
    }

    /// Ios 일 때
    else {
      flutterLocalNotificationsPlugin.resolvePlatformSpecificImplementation<IOSFlutterLocalNotificationsPlugin>()?.requestPermissions(
        alert: true,
        badge: true,
        sound: true,
      );
    }
  }
}