import 'package:flutter/material.dart';
import 'package:get/get.dart';

import '../../bottom_navigator.dart';
import '../../components/dialog.dart';
import '../../provider/notification_state.dart';
import '../../screen/util/font/font.dart';

class AlertTurnOff extends StatefulWidget {
  const AlertTurnOff({Key? key}) : super(key: key);

  @override
  State<AlertTurnOff> createState() => _AlertTurnOffState();
}

class _AlertTurnOffState extends State<AlertTurnOff> {
  final ns = Get.put(NotificationState());
  TextEditingController _reasonCon = TextEditingController();
  List _alertTitle = [];
  int? _selectedIndex;

  @override
  void initState() {
    _alertTitle = ns.alertTurnOffList.value;

    super.initState();
  }

  @override
  void dispose() {
    _reasonCon.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
        FocusScope.of(context).unfocus();
      },
      child: Scaffold(
          appBar: AppBar(
            title: Text(
              '알림해제',
              style: f16w900Size(),
            ),
            centerTitle: true,
            shape: Border(
              bottom: BorderSide(
                color: const Color(0xffEFF0F0),
                width: 1,
              ),
            ),
          ),
          backgroundColor: const Color(0xffF1F4F7),
          body: Padding(
            padding: const EdgeInsets.symmetric(horizontal: 10, vertical: 10),
            child: SingleChildScrollView(
              physics: const ClampingScrollPhysics(),
              child: Column(
                children: [
                  ListView.builder(
                      itemCount: _alertTitle.length,
                      shrinkWrap: true,
                      physics: const ClampingScrollPhysics(),
                      itemBuilder: (_, index) {
                        // bool _isLastIndex = index == _alertTitle.length - 1;
                        return GestureDetector(
                          onTap: () {
                            // _isLastIndex = index == _alertTitle.length - 1;
                            _selectedIndex = index;
                            setState(() {});
                          },
                          child: Container(
                            margin: const EdgeInsets.only(bottom: 10),
                            decoration: BoxDecoration(
                              borderRadius: BorderRadius.circular(20),
                              border: Border.all(color: _selectedIndex == index ? const Color(0xffE83B3B) : Colors.transparent),
                              color: Colors.white,
                            ),
                            padding: const EdgeInsets.symmetric(horizontal: 10, vertical: 20),
                            width: Get.width,
                            child: Text(
                              '${_alertTitle[index]}',
                              style: f20w500Size(),
                              textAlign: TextAlign.center,
                            ),
                          ),
                        );
                      }),
                  AnimatedContainer(
                    width: _selectedIndex == _alertTitle.length - 1 ? Get.width : 0,
                    height: _selectedIndex == _alertTitle.length - 1 ? 160 : 0,
                    duration: Duration(milliseconds: 500),
                    curve: Curves.fastOutSlowIn,
                    child:  TextFormField(
                      controller: _reasonCon,
                      minLines: 3,
                      maxLines: 3,
                      decoration: InputDecoration(
                        hintText: '사유를 입력해주세요',
                        hintStyle: hintf14w400Size(),
                        // contentPadding: EdgeInsets.symmetric(horizontal: 10, vertical: 16),
                        filled: true,
                        fillColor: Colors.white,
                        enabledBorder: OutlineInputBorder(
                            borderSide: BorderSide(
                              color: _selectedIndex == _alertTitle.length - 1 ? Color(0xffE83B3B) : Colors.transparent,
                              width: 1.0,
                            ),
                          borderRadius: BorderRadius.circular(12)
                        ),
                        focusedBorder: OutlineInputBorder(
                            borderSide: BorderSide(
                              color: _selectedIndex == _alertTitle.length - 1 ? Color(0xffE83B3B) : Colors.transparent,
                              width: 1.0,
                            ),
                            borderRadius: BorderRadius.circular(12)
                        ),
                      ),
                    ),
                    // child: Text(
                    //   '직접 입력',
                    //   style: f20w500Size(),
                    //   textAlign: TextAlign.center,
                    // ),
                  ),
                ],
              ),
            ),
          ),
          bottomNavigationBar: GestureDetector(
            onTap: () async {
              showOnlyConfirmTapDialog(context, '해제를 했습니다', () {
                Get.offAll(BottomNavigator());
              });
            },
            child: Container(
              width: Get.width,
              height: 60,
              color: _selectedIndex == null ? const Color(0xffD3D8DE) : const Color(0xffE83B3B),
              padding: const EdgeInsets.symmetric(vertical: 16),
              child: Center(
                  child: Text(
                '선택',
                style: f18w700WhiteSize(),
              )),
            ),
          )),
    );
  }
}
