import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import '../../notification/alert/alert_camera.dart';
import '../../notification/alert/alert_collecting_well.dart';
import '../../notification/alert/alert_fire_receiver.dart';
import '../../notification/alert/alert_septic_tank.dart';
import '../../notification/alert/alert_transformer.dart';
import '../../notification/alert/alert_water_tank.dart';
import '../util/font/font.dart';
import 'package:http/http.dart' as http;

import '../util/loading.dart';

class AlimScreen extends StatefulWidget {
  const AlimScreen({Key? key}) : super(key: key);

  @override
  State<AlimScreen> createState() => _AlimScreenState();
}

class _AlimScreenState extends State<AlimScreen>
    with SingleTickerProviderStateMixin {
  TabController? _tabController;
  late List<Map<String, dynamic>> alimData;
  bool isLoading = true;
  /// 데이터 가져오기
  Future<void> fetchData() async {
    try {
      final response = await http.get(Uri.parse('http://192.168.0.3:6543/senddata?id=${us.userList[0]['mms']}'));
      if (response.statusCode == 200) {
        setState(() {
          alimData = List<Map<String, dynamic>>.from(json.decode(response.body));
        });
      } else {
        print('서버 응답 실패: ${response.reasonPhrase}');

      }
    } catch (e) {
      print('에러 발생: $e');
    }
  }

  @override
  void initState() {
    _tabController = TabController(length: 3, vsync: this);
    // _tabController = TabController(length: 1, vsync: this);
    Future.delayed(Duration.zero,()async{
      await fetchData();
      isLoading = false;
      setState(() {});
    });
    super.initState();
  }

  @override
  void dispose() {
    _tabController?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        automaticallyImplyLeading: false,
        titleSpacing: 0,
        bottom: PreferredSize(
          preferredSize: Size.fromHeight(10),
          child: Container(
            width: Get.width,
            decoration: BoxDecoration(
              border: Border(
                bottom: BorderSide(width: 1.0, color: Color(0xffF1F4F7)),
              ),
            ),
            child: Padding(
              padding: const EdgeInsets.only(left: 20),
              child: Align(
                alignment: Alignment.centerLeft,
                child: TabBar(
                  isScrollable: true,
                  controller: _tabController,
                  unselectedLabelColor: Color(0xffAEB1B9),
                  unselectedLabelStyle: f16w500HintSize(),
                  labelStyle: f16w900Size(),
                  overlayColor:
                      MaterialStateProperty.all<Color>(Colors.transparent),
                  dividerColor: Color(0xffF1F4F7),
                  labelPadding: EdgeInsets.only(bottom: 10, right: 25),
                  indicator: UnderlineTabIndicator(
                    borderSide: BorderSide(color: Colors.black, width: 3),
                  ),
                  labelColor: Colors.black,
                  tabs: [
                    Tab(text: '전체'),
                    Tab(text: '모니터링'),
                    Tab(text: '카메라'),
                  ],
                ),
              ),
            ),
          ),
        ),
      ),
      body: isLoading?LoadingScreen():
      TabBarView(
        controller: _tabController,
        children: [
          /// 전체 알림
          ListView.builder(
              shrinkWrap: true,
              physics: const ClampingScrollPhysics(),
              itemCount: alimData.length,
              itemBuilder: (_, index) {
                Map<String, dynamic> item = alimData[index];
                String notificationType =
                    int.parse(item['type'].toString()) == 0
                        ? '모니터링 알림'
                        : '카메라 알림';
                String title = item['title'] ?? '';
                String createDate = item['createDate'] ?? '';
                return Container(
                  padding: EdgeInsets.symmetric(vertical: 14, horizontal: 20),
                  decoration: BoxDecoration(
                      border:
                          Border(bottom: BorderSide(color: Color(0xffF1F4F7)))),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text(
                        notificationType,

                        /// type 0 = 모니터링 알림 or type 1 = 카메라 알림
                        style: f15w600Blue(),
                      ),
                      // Spacer(),
                      Text(
                        title,

                        /// title
                        style: f18w700Size(),
                      ),
                      Text(
                        createDate,

                        /// createDate
                        style: f15w600Grey(),
                      ),
                    ],
                  ),
                );
              }),

          /// 모니터링 알림
          ListView.builder(
              shrinkWrap: true,
              physics: const ClampingScrollPhysics(),
              itemCount: alimData.length,
              itemBuilder: (_, index) {
                Map<String, dynamic> item = alimData[index];
                if (int.parse(item['type'].toString()) == 0) {
                  String notificationType = '모니터링 알림';
                  String title = item['title'] ?? '';
                  String createDate = item['createDate'] ?? '';

                  return GestureDetector(
                    onTap: (){
                      // Get.to(() => AlertWaterTank());
                      // Get.to(() => AlertCollectingWell());
                      // Get.to(() => AlertFireReceiver());
                      // Get.to(() => AlertTransFormer());
                      //  Get.to(() => AlertSepticTank());
                       Get.to(() => AlertCamera());
                    },
                    child: Container(
                      padding: EdgeInsets.symmetric(vertical: 14, horizontal: 20),
                      decoration: BoxDecoration(
                          border: Border(
                              bottom: BorderSide(color: Color(0xffF1F4F7)))),
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          Text(
                            notificationType,

                            /// type 0 = 모니터링 알림 or type 1 = 카메라 알림
                            style: f15w600Blue(),
                          ),
                          // Spacer(),
                          Text(
                            title,

                            /// title
                            style: f18w700Size(),
                          ),
                          Text(
                            createDate,

                            /// createDate
                            style: f15w600Grey(),
                          ),
                        ],
                      ),
                    ),
                  );
                } else {
                  return SizedBox.shrink();
                }
              }),

          /// 카메라 알림
          ListView.builder(
              shrinkWrap: true,
              physics: const ClampingScrollPhysics(),
              itemCount: alimData.length,
              itemBuilder: (_, index) {
                Map<String, dynamic> item = alimData[index];
                if (int.parse(item['type'].toString()) == 1) {
                  String notificationType = '카메라 알림';
                  String title = item['title'] ?? '';
                  String createDate = item['createDate'] ?? '';

                  return Container(
                    padding: EdgeInsets.symmetric(vertical: 14, horizontal: 20),
                    decoration: BoxDecoration(
                        border: Border(
                            bottom: BorderSide(color: Color(0xffF1F4F7)))),
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        Text(
                          notificationType,

                          /// type 0 = 모니터링 알림 or type 1 = 카메라 알림
                          style: f15w600Blue(),
                        ),
                        // Spacer(),
                        Text(
                          title,

                          /// title
                          style: f18w700Size(),
                        ),
                        Text(
                          createDate,

                          /// createDate
                          style: f15w600Grey(),
                        ),
                      ],
                    ),
                  );
                } else {
                  return SizedBox.shrink();
                }
              }),
        ],
      ),
    );
  }
}
