import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:get/get.dart';
import 'package:get_ip_address/get_ip_address.dart';
import 'package:intl/intl.dart';
import 'package:vsdk_example/play/play_logic.dart';
import 'package:vsdk_example/utils/super_put_controller.dart';
import '../../../../main/main_logic.dart';
import '../../bottom_navigator.dart';
import '../../components/dialog.dart';
import '../../provider/user_state.dart';
import '../camera/camera_main.dart';
import '../util/encryption.dart';
import '../util/font/font.dart';
import 'admin_confirm_screen.dart';
import 'package:http/http.dart' as http;

class LoginName extends StatefulWidget {
  const LoginName({super.key});

  @override
  State<LoginName> createState() => _LoginNameState();
}

class _LoginNameState extends State<LoginName> {
  final us = Get.put(UserState());
  final FocusNode _emailFocusNode = FocusNode();
  final FocusNode _passwordFocusNode = FocusNode();
  final FocusNode _loginButtonFocusNode = FocusNode();
  TextEditingController _idCon = TextEditingController();
  TextEditingController _pwCon = TextEditingController();
  bool isChecked = false;
  bool loginCheck = false;
  bool _obscurePassword = true;
  final storage = new FlutterSecureStorage();
  String? fontSizes;

  @override
  void initState() {
    Future.delayed(Duration.zero, () async {
      fontSizes = (await storage.read(key: "fontSizes"));
      switch (fontSizes) {
        case '2':
          us.userFont.value = 2;
          break;
        case '1':
          us.userFont.value = 1;
          break;
        case '0':
          us.userFont.value = 0;
          break;
        default:
          us.userFont.value = 0;
          break;
      }
      setState(() {});
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
        FocusScope.of(context).unfocus();
      },
      child: Scaffold(
        body: SingleChildScrollView(
          physics: const ClampingScrollPhysics(),
          child: Container(
            height: Get.height,
            child: Padding(
              padding: const EdgeInsets.symmetric(horizontal: 20),
              child: Column(
                mainAxisAlignment: MainAxisAlignment.center,
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  SvgPicture.asset(
                    'assets/icon/logo.svg',
                  ),
                  const SizedBox(height: 60),
                  Text(
                    '아이디',
                    style: f16w700Size(),
                  ),
                  const SizedBox(
                    height: 10,
                  ),
                  TextFormField(
                    onFieldSubmitted: (value) {
                      FocusScope.of(context).requestFocus(_passwordFocusNode);
                    },
                    focusNode: _emailFocusNode,
                    controller: _idCon,
                    decoration: InputDecoration(
                      hintText: '이메일을 입력해주세요',
                      hintStyle: hintf14w400Size(),
                      contentPadding: EdgeInsets.symmetric(horizontal: 15, vertical: 16),
                      filled: true,
                      fillColor: Color(0xFFF1F4F7),
                      enabledBorder: UnderlineInputBorder(
                        borderRadius: BorderRadius.circular(4),
                        borderSide: BorderSide.none,
                      ),
                      focusedBorder: UnderlineInputBorder(
                        borderRadius: BorderRadius.circular(4),
                        borderSide: BorderSide.none,
                      ),
                    ),
                  ),

                  ///이메일 입력칸

                  const SizedBox(
                    height: 20,
                  ),

                  Text(
                    '비밀번호',
                    style: f16w700Size(),
                  ),

                  const SizedBox(
                    height: 10,
                  ),
                  TextFormField(
                    onFieldSubmitted: (value) {
                      FocusScope.of(context).requestFocus(_loginButtonFocusNode);
                    },
                    focusNode: _passwordFocusNode,
                    controller: _pwCon,
                    obscureText: _obscurePassword,
                    decoration: InputDecoration(
                      hintText: '비밀번호를 입력해주세요',
                      hintStyle: hintf14w400Size(),
                      contentPadding: EdgeInsets.symmetric(horizontal: 15, vertical: 16),
                      filled: true,
                      fillColor: Color(0xFFF1F4F7),
                      enabledBorder: UnderlineInputBorder(
                        borderRadius: BorderRadius.circular(4),
                        borderSide: BorderSide.none,
                      ),
                      focusedBorder: UnderlineInputBorder(
                        borderRadius: BorderRadius.circular(4),
                        borderSide: BorderSide.none,
                      ),
                      suffixIcon: GestureDetector(
                        onTap: () {
                          setState(() {
                            _obscurePassword = !_obscurePassword;
                          });
                        },
                        child: Icon(
                          _obscurePassword ? Icons.visibility_off : Icons.visibility,
                          color: Color(0xff999FAF),
                        ),
                      ),
                    ),
                  ),

                  ///비밀번호 입력칸

                  const SizedBox(
                    height: 20,
                  ),

                  Row(
                    children: [
                      GestureDetector(
                        onTap: () {
                          setState(() {
                            isChecked = !isChecked;
                          });
                        },
                        child: SvgPicture.asset(
                          isChecked ? 'assets/icon/check.svg' : 'assets/icon/uncheck.svg',
                          width: 21,
                          height: 21,
                        ),
                      ),
                      SizedBox(width: 10),
                      Text(
                        '아이디 저장',
                        style: f14w700Size(),
                      ),
                    ],
                  ),

                  const SizedBox(
                    height: 30,
                  ),

                  Center(
                    child: ElevatedButton(
                      focusNode: _loginButtonFocusNode,
                      style: ButtonStyle(
                        shape: MaterialStateProperty.all<RoundedRectangleBorder>(
                          RoundedRectangleBorder(
                            borderRadius: BorderRadius.circular(4),
                          ),
                        ),
                        minimumSize: MaterialStateProperty.all<Size>(
                          Size(double.infinity, 52),
                        ),
                        backgroundColor: MaterialStateProperty.all<Color>(
                          Color(0xFF1955EE),
                        ),
                      ),
                      onPressed: () async {
                        // await getUserInfo(_idCon.text, _pwCon.text);
                        // if (loginCheck) {
                        //   if (us.userList[0]['first'] == 'false') {
                        //     await MonitoringInfo();
                        //     Get.offAll(BottomNavigator());
                        //     // Get.to(() => BottomNavigator());
                        //   } else {
                        //     await MonitoringInfo();
                        //     Get.to(() => AdminConfirm());
                        //   }
                        // } else {
                        //   showOnlyConfirmDialog(context, '아이디 또는 비밀번호가 틀립니다');
                        //   print('로그인안됨');
                        // }
                        Get.to(() => CameraMain());
                      },
                      child: Text('로그인', style: f16w700WhiteSize()),
                    ),
                  ),

                  SizedBox(
                    height: 20,
                  ),

                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      Text(
                        '아이디 찾기',
                        style: hintf14w400Size(),
                      ),
                      SizedBox(
                        width: 42,
                      ),
                      Text(
                        '|',
                        style: hintf14w400Size(),
                      ),
                      SizedBox(
                        width: 42,
                      ),
                      Text(
                        '비밀번호 찾기',
                        style: hintf14w400Size(),
                      ),
                    ],
                  ),
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }

  /// ip 가져오기
  Future<String?> getPublicIP() async {
    final us = Get.put(UserState());
    try {
      /// Initialize Ip Address
      var ipAddress = IpAddress(type: RequestType.json);

      /// Get the IpAddress based on requestType.
      dynamic data = await ipAddress.getIpAddress();
      us.usipAddress.value = data['ip'];
      print('현재 ip : ${us.usipAddress.value}');
    } on IpAddressException catch (exception) {
      /// Handle the exception.
      // print(exception.message);
    }
  }

  /// 로그 추가히기
  Future<void> logAdd(String id) async {
    final us = Get.put(UserState());

    final url =
        'http://misnetwork.iptime.org:9000/logAdd?id=${id}&ip=${us.usipAddress.value}&createDate=${DateFormat('yyyy-MM-dd HH:mm:ss').format(DateTime.parse('${DateTime.now()}'))}';
    final response = await http.get(Uri.parse(url));
    print(response.body);

    if (response.statusCode != 200) {
      print('에러에러');
      // showOnlyConfirmDialog(context, '인증번호 요청에 실패하였습니다. 다시 시도해주세요');
      throw Exception('Failed to send email');
    }
  }

  /// 회원정보 가져오기
  Future<void> getUserInfo(String id, String pw) async {
    final us = Get.put(UserState());
    final url = 'http://misnetwork.iptime.org:9000/login?id=${id}&pw=${pw}';
    final response = await http.get(Uri.parse(url));
    List<dynamic> dataList = json.decode(response.body);
    us.userList.value = dataList;
    if (us.userList.isEmpty) {
      setState(() {});
      print('로그인정보 없음');
    } else {
      if (textEncryption(us.userList[0]['pw'].toString().replaceAll(' ', '+'), 0) == pw) {
        loginCheck = true;
        setState(() {});
      } else {
        loginCheck = false;
        setState(() {});
      }
    }
    if (response.statusCode != 200) {
      print('에러에러');
      throw Exception('Failed to send email');
    }
  }

  /// 모니터링 정보 가져오기
  Future<void> MonitoringInfo() async {
    final us = Get.put(UserState());
    final url = 'http://misnetwork.iptime.org:9000/monitoInfo?id=${us.userList[0]['mms']}';
    final response = await http.get(Uri.parse(url));
    Map<String, dynamic> dataList = json.decode(response.body);
    us.userMonitoring.value = [dataList];
    if (response.statusCode != 200) {
      throw Exception('Failed to send email');
    }
  }
}
