import 'dart:convert';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import '../../../components/color_radio.dart';
import '../../../components/switch.dart';
import '../../../components/switch_container.dart';
import '../../../db/get_monitoring_info.dart';
import '../../../provider/user_state.dart';
import '../../util/font/font.dart';

class FireSettingScreen extends StatefulWidget {
  const FireSettingScreen({Key? key}) : super(key: key);

  @override
  State<FireSettingScreen> createState() => _FireSettingScreenState();
}

class _FireSettingScreenState extends State<FireSettingScreen> {
  final us = Get.put(UserState());
  List item = ['알림발송','주경종','지구경종','부저','사이렌'];
  List field = ['alim','mainjong','subjong','buzzer','siren'];
  List alimCheck = [];

  late Socket socket;
  bool _isLoading = true;


  String decimalToHex(int decimalNumber) {
    String hex = decimalNumber.toRadixString(16).toUpperCase();
    if (hex.length == 1) {
      hex = '0' + hex;
    }
    return hex;
  }

  @override
  void initState() {

    Future.delayed(Duration.zero,()async{
      await MonitoringInfo();
      socket = await Socket.connect('220.89.248.140', 5161);
      await getInfo();
      socket.add(us.userSocketData);
      // socket.listen((List<int> data) {
      //   String message = String.fromCharCodes(data).trim();
      //   print('수신된 데이터: $message');
      // },
      //   onDone: () {
      //     print('서버와의 연결이 종료되었습니다.');
      //     socket.destroy();
      //   },
      // );
      // await getInfo();
      _isLoading = false;
      setState(() {});
    });
    super.initState();
  }
  @override
  void dispose() {
    socket.close();
    socket.destroy();
    super.dispose();
  }
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(
          '소방수신기 상태',
          style: f16w800Size(),
        ),
        titleSpacing: -15,
        centerTitle: true,
        actions: [
          Padding(
            padding: const EdgeInsets.only(right:12),
            child: Row(
              children: [
                RedRadio(),
                const SizedBox(width: 6,),
                Text('불량',style: redf18w700(),),
              ],
            ),
          )
        ],
        shape: Border(
          bottom: BorderSide(
            color: Color(0xffEFF0F0),
            width: 1,
          ),
        ),
      ),
      body: _isLoading?Container():StreamBuilder(
        stream: socket,
        builder: (context,snapshot){
          if (snapshot.connectionState == ConnectionState.waiting || !snapshot.hasData) {
            return Container(width: Get.width,height: Get.height, child: Container());
          }
          else{
            // List<dynamic> dataList = jsonDecode(String.fromCharCodes(snapshot.data!));
            List<dynamic> dataList = [];
            print('data check : ${snapshot.data}');


            if (dataList.isNotEmpty) {
              return Container(
                height: Get.height,
                color: Color(0xffF1F4F7),
                child: ListView.builder(
                    shrinkWrap: true,
                    physics: const ClampingScrollPhysics(),

                    itemCount: item.length,
                    itemBuilder: (_, index) {
                      return Padding(
                        padding: const EdgeInsets.only(left: 20,right: 20),
                        child: Column(
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: [
                            index ==0?SizedBox(height: 10,):SizedBox(),
                            SwitchContainer(
                                onTap: ()async{
                                  /// 알람 주경종 지구경종 사이렌 부저
                                    await MonitoringInfo(); /// db에서 정보 가져오기관
                                    if(bool.parse('${us.userMonitoring[0][field[index]]}')){
                                      us.userMonitoring[0][field[index]] = false;
                                    }
                                    else{
                                      us.userMonitoring[0][field[index]] = true;
                                    }
                                    print('현재 넣는 값?? ${bool.parse('${us.userMonitoring[0][field[index]]}')}');
                                    await changeSwitch(bool.parse('${us.userMonitoring[0][field[index]]}'),'${item[index]}');
                                    socket.add(us.userSocketData);
                                    // await updateButton(alimCheck[index]);
                                },
                                value: dataList[0][field[index]] =='0'?false:bool.parse(dataList[0][field[index]]),
                                name: '${item[index]}',
                                value2:dataList[0][field[index]]),
                            // Row(
                            //   mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            //   children: [
                            //     Text('${item[index]}',style: f24w700Size(),),
                            //     dataList[0][field[index]] =='0'
                            //         ?Text('미등록',style: f24w700Size(),)
                            //         :SwitchButton(onTap: ()async{
                            //           alimCheck[index] = bool.parse(dataList[0][field[index]]);
                            //           alimCheck[index] = !alimCheck[index];
                            //           print('필드 :: ${field[index]} 값 ${alimCheck[index]}');
                            //          await updateButton(field[index], alimCheck[index]);
                            //     }, value: bool.parse(dataList[0][field[index]]))
                            //   ],
                            // ),
                            const SizedBox(height: 10,)
                          ],
                        ),
                      );
                    }),
              );
            }
            else{
              return Container(width: Get.width, height: Get.height, child: Container());
            }
          }
        },
      )
    );
  }
  // Future<void> updateButton(bool value) async {
  //   /// 0x05 인데 내가 하나를 바꿧어
  //   /// index = 0이 true로 바뀌면 0001
  //   /// index = 1 이 true로 바뀌면 0010
  //   /// index = 2 이면 1000
  //   /// index =3 이면 0100
  //   /// 예를들어 스위치가 인덱스 1번하고 3번이 켜져있으면 0110 이렇게 나오게 해줘
  //
  //  String switchString = alimCheck.map((flag) => flag?'1':'0').join();
  //  String reversedString = switchString.substring(1).split('').reversed.join();
  //
  //  int decimalNumber = 30;
  //  String hexadecimalNumber = decimalToHex(decimalNumber);
  //  String hexString = '0x$hexadecimalNumber';
  //
  //  int decimalValue = int.parse(reversedString, radix: 2);
  //  String hexvalue = '0x${decimalValue.toRadixString(16).padLeft(2, '0')}';
  //
  //  dataToSend = [
  //    0xc0, // Start
  //    0x0d, // Len
  //    0x10, // Packet Type
  //    0x05, 0x01, 0x00, 0x23, 0x05, int.parse('${hexvalue}'), 0x1a, 0xb0, 0x00, 0x00, int.parse('${hexString}'), //Payload
  //    0xa8 // XOR
  //  ];
  //  socket.add(dataToSend);
  //
  // }
}
