import 'dart:async';
import 'dart:io';
import 'package:dropdown_button2/dropdown_button2.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:get/get.dart';
import '../../components/color_radio.dart';
import '../../components/dialog.dart';
import '../../db/get_monitoring_info.dart';
import '../../provider/user_state.dart';
import '../login/login_name_screen.dart';
import '../util/font/font.dart';
import 'fire/fire_setting_screen.dart';

class MonitoringMainPage extends StatefulWidget {
  const MonitoringMainPage({Key? key}) : super(key: key);

  @override
  State<MonitoringMainPage> createState() => _MonitoringMainPageState();
}

class _MonitoringMainPageState extends State<MonitoringMainPage> {
  final us = Get.put(UserState());

  /// drop
  String? _selectedValue = '김나다 아파트';
  final List<String> _items = ['김나다 아파트', '떙떙 아파트', '아파트으'];

  late Socket socket;
  bool _isLoading = true;
  List alimCheck = [];

  Timer? _timer;

  /// 급수관 개폐 함수
  bool switchButton = false;

  @override
  void initState() {
    Future.delayed(Duration.zero, () {
      alimCheck = List.generate(5, (index) => false);
      _timer = Timer.periodic(Duration(seconds: 2), (timer) async {
        try {
          await MonitoringInfo();
          switchButton = bool.parse('${us.userMonitoring[0]['data']}');
          _isLoading = false;
          setState(() {});
        } catch (e) {
          _timer?.cancel();
          showOnlyConfirmTapDialog(context, '서버가 종료되었습니다.', () {
            Get.offAll(() => LoginName());
          });
        }
      });
    });
    super.initState();
  }

  @override
  void dispose() {
    _timer?.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: PreferredSize(
        preferredSize: Size.fromHeight(52),
        child: AppBar(
          automaticallyImplyLeading: false,
          actions: [
            Text(
              '${us.userMonitoring[0]['agency']}',
              style: f16w700Size(),
            ),
            // Container(
            //   width: Get.width,
            //   padding: const EdgeInsets.symmetric(vertical: 14),
            //   child: DropdownButtonHideUnderline(
            //     child: DropdownButton2<String>(
            //       isExpanded: true,
            //       items: _items
            //           .map((String item) => DropdownMenuItem<String>(
            //                 value: item,
            //                 child: Text('${item}', style: f20w700Size()),
            //               ))
            //           .toList(),
            //       value: _selectedValue,
            //       onChanged: (String? value) {
            //         setState(() {
            //           _selectedValue = value;
            //         });
            //       },
            //       iconStyleData: const IconStyleData(
            //         icon: Icon(
            //           Icons.keyboard_arrow_down_outlined,
            //         ),
            //         iconSize: 20,
            //         iconEnabledColor: Colors.black,
            //         iconDisabledColor: Colors.black,
            //       ),
            //       buttonStyleData: const ButtonStyleData(
            //         padding: EdgeInsets.symmetric(horizontal: 24),
            //         height: 40,
            //         width: 140,
            //       ),
            //       dropdownStyleData: DropdownStyleData(
            //         maxHeight: 200,
            //         decoration: BoxDecoration(),
            //         offset: const Offset(0, -10),
            //         scrollbarTheme: ScrollbarThemeData(
            //           radius: const Radius.circular(40),
            //           thickness: MaterialStateProperty.all(6),
            //           thumbVisibility: MaterialStateProperty.all(true),
            //         ),
            //       ),
            //     ),
            //   ),
            // ),
          ],
        ),
      ),
      body: _isLoading
          ? Center(
              child: CircularProgressIndicator(),
            )
          : SingleChildScrollView(
              physics: const ClampingScrollPhysics(),
              child: Container(
                color: Color(0xffF1F4F7),
                child: Padding(
                  padding: const EdgeInsets.only(left: 10, right: 10, top: 20),
                  child: Column(
                    children: [
                      /// 저수조 수위
                      /// 29% 이하 또는 69% 이상일 경우에는 붉은 글씨
                      /// 오뚜기는 물 부족 경고 와 불 넘침 경고
                      Stack(
                        children: [
                          Positioned(
                            child: Container(
                                width: Get.width,
                                height: 135,
                                child: SvgPicture.asset(
                                  'assets/icon/water.svg',
                                  fit: BoxFit.fill,
                                )),
                          ),
                          Positioned.fill(
                            top: 20,
                            child: Align(
                                alignment: Alignment.topCenter,
                                child: Text(
                                  '저수조 수위',
                                  style: f18w700Size(),
                                )),
                          ),
                          Positioned.fill(
                            left: 20,
                            right: 20,
                            child: Column(
                              mainAxisAlignment: MainAxisAlignment.center,
                              crossAxisAlignment: CrossAxisAlignment.center,
                              children: [
                                Row(
                                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                                  children: [
                                    Text(
                                      '디지털계량기',
                                      style: f16w700Size(),
                                    ),
                                    Text(
                                      '오뚜기 센서',
                                      style: f16w700Size(),
                                    ),
                                  ],
                                ),
                                const SizedBox(
                                  height: 10,
                                ),
                              ],
                            ),
                          ),
                          Positioned.fill(
                            left: 30, // 3자리면 28
                            right: 32, // 정상 right 32 비정상 right 22
                            bottom: 20,
                            child: Column(
                              mainAxisAlignment: MainAxisAlignment.end,
                              crossAxisAlignment: CrossAxisAlignment.center,
                              children: [
                                Row(
                                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                                  children: [
                                    Text(
                                      '${us.userMonitoring[0]['waterLevel']}%',
                                      style:
                                          int.parse(us.userMonitoring[0]['waterLevel']) <= 29 || int.parse(us.userMonitoring[0]['waterLevel']) >= 69
                                              ? f28w700RedSize()
                                              : f28w700Size(),
                                    ),
                                    Text(
                                      us.userMonitoring[0]['ottogi'] == '0'
                                          ? '정상'
                                          : us.userMonitoring[0]['ottogi'] == '1'
                                              ? '물부족'
                                              : us.userMonitoring[0]['ottogi'] == '2'
                                                  ? '물넘침'
                                                  : '센서없음',
                                      style: us.userMonitoring[0]['ottogi'] == '0' ? f28w700Size() : f28w700RedSize(),
                                    ),
                                  ],
                                )
                              ],
                            ),
                          )
                        ],
                      ),
                      const SizedBox(
                        height: 10,
                      ),

                      /// 집수정
                      Container(
                        width: Get.width,
                        height: 75,
                        decoration: BoxDecoration(color: Colors.white, borderRadius: BorderRadius.circular(20)),
                        child: Padding(
                          padding: const EdgeInsets.only(left: 20, right: 18),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.start,
                            children: [
                              Text(
                                '집수정',
                                style: f18w700Size(),
                              ),
                              const Spacer(),
                              us.userMonitoring[0]['jibsu'] == '0' ? BlueRadio() : RedRadio(),
                              const SizedBox(
                                width: 6,
                              ),
                              Text(
                                us.userMonitoring[0]['jibsu'] == '0' ? '정상' : '고수위',
                                style: us.userMonitoring[0]['jibsu'] == '0' ? bluef18w700() : redf18w700(),
                              ),
                              const SizedBox(
                                width: 14,
                              ),
                            ],
                          ),
                        ),
                      ),
                      const SizedBox(
                        height: 10,
                      ),

                      /// 급수관 개폐
                      Container(
                          padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 20),
                          decoration: BoxDecoration(borderRadius: BorderRadius.circular(20), color: Colors.white),
                          width: Get.width,
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              Text(
                                '급수밸브',
                                style: f14w600Size(),
                              ),
                              SizedBox(height: 10),
                              Row(
                                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                                children: [
                                  GestureDetector(
                                      onTap: () async {
                                        switchButton = true;
                                        await changeSwitch(switchButton, '급수관');
                                        socket = await Socket.connect('192.168.0.21', 5161);
                                        socket.add(us.userSocketData.value);
                                        socket.close();
                                        setState(() {});
                                      },
                                      child: Text(
                                        '열기',
                                        style: switchButton ? f16w700Size() : f16w600GreySize(),
                                      )),
                                  switchButton
                                      ? SvgPicture.asset(
                                          'assets/icon/valve_open.svg',
                                          width: 140,
                                          height: 48,
                                        )
                                      : SvgPicture.asset(
                                          'assets/icon/valve_closed.svg',
                                          width: 140,
                                          height: 48,
                                        ),
                                  GestureDetector(
                                      onTap: () async {
                                        switchButton = false;
                                        await changeSwitch(switchButton, '급수관');
                                        socket = await Socket.connect('192.168.0.21', 5161);
                                        socket.add(us.userSocketData.value);
                                        socket.close();
                                        setState(() {});
                                      },
                                      child: Text(
                                        '닫기',
                                        style: !switchButton ? f16w700Size() : f16w600GreySize(),
                                      )),
                                ],
                              ),
                            ],
                          )),
                      const SizedBox(
                        height: 10,
                      ),
                      const SizedBox(
                        height: 10,
                      ),
                      GestureDetector(
                        behavior: HitTestBehavior.opaque,
                        onTap: () {
                          Get.to(() => FireSettingScreen());
                        },
                        child: Container(
                          width: Get.width,
                          height: 75,
                          decoration: BoxDecoration(color: Colors.white, borderRadius: BorderRadius.circular(20)),
                          child: Padding(
                            padding: const EdgeInsets.only(left: 20, right: 18),
                            child: Row(
                              mainAxisAlignment: MainAxisAlignment.start,
                              children: [
                                Text(
                                  '소방수신기 상태',
                                  style: f18w700Size(),
                                ),
                                Spacer(),
                                us.userMonitoring[0]['firestate'] == '0' ? BlueRadio() : RedRadio(),
                                const SizedBox(
                                  width: 6,
                                ),
                                Text(
                                  us.userMonitoring[0]['firestate'] == '0' ? '정상' : '불량',
                                  style: us.userMonitoring[0]['firestate'] == '0' ? bluef18w700() : redf18w700(),
                                ),
                                const SizedBox(
                                  width: 14,
                                ),
                                SvgPicture.asset('assets/icon/rightArrow.svg')
                              ],
                            ),
                          ),
                        ),
                      ),
                      const SizedBox(
                        height: 32,
                      ),

                      /// 변압기 온도 , 변압기 주변 온도
                      Container(
                        width: Get.width,
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.center,
                          children: [
                            const Spacer(),
                            Container(
                              child: Column(
                                crossAxisAlignment: CrossAxisAlignment.center,
                                children: [
                                  Text(
                                    '변압기 온도',
                                    style: grf14w600(),
                                  ),
                                  Row(
                                    children: [
                                      Text(
                                        '${us.userMonitoring[0]['tempMain']}',
                                        style: f28w700Size(),
                                      ),
                                      const SizedBox(
                                        width: 4,
                                      ),
                                      Text(
                                        '°C',
                                        style: f18w400Size(),
                                      )
                                    ],
                                  )
                                ],
                              ),
                            ),
                            const Spacer(),
                            Container(
                              height: 38,
                              child:   const VerticalDivider(
                                color:   const Color(0xffC5C9CC),
                                thickness: 1,
                              ),
                            ),
                            const Spacer(),
                            Container(
                              child: Column(
                                children: [
                                  Text(
                                    '변압기 주변 온도',
                                    style: grf14w600(),
                                  ),
                                  Row(
                                    children: [
                                      Text(
                                        '${us.userMonitoring[0]['tempside']}',
                                        style: f28w700Size(),
                                      ),
                                      const SizedBox(
                                        width: 4,
                                      ),
                                      Text(
                                        '°C',
                                        style: f18w400Size(),
                                      )
                                    ],
                                  )
                                ],
                              ),
                            ),
                            const Spacer(),
                          ],
                        ),
                      ),

                      const SizedBox(
                        height: 10,
                      ),

                      /// 정화조 수위
                      Container(
                        width: Get.width,
                        height: 75,
                        decoration: BoxDecoration(color: Colors.white, borderRadius: BorderRadius.circular(20)),
                        child: Padding(
                          padding: const EdgeInsets.only(left: 20, right: 18),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.start,
                            children: [
                              Text(
                                '정화조 수위',
                                style: f18w700Size(),
                              ),
                              const Spacer(),
                              us.userMonitoring[0]['cleanLevel'] == '1' ? BlueRadio() : RedRadio(),
                              const SizedBox(
                                width: 6,
                              ),
                              Text(
                                us.userMonitoring[0]['cleanLevel'] == '1' ? '정상' : '오수 넘침 경고',
                                style: us.userMonitoring[0]['cleanLevel'] == '1' ? bluef18w700() : redf18w700(),
                              ),
                              const SizedBox(
                                width: 14,
                              ),
                            ],
                          ),
                        ),
                      ),
                      const SizedBox(
                        height: 10,
                      ),

                    ],
                  ),
                ),
              ),
            ),
    );
  }

}
