import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:get/get.dart';
import 'package:mask_text_input_formatter/mask_text_input_formatter.dart';
import '../../../components/dialog.dart';
import '../../../provider/user_state.dart';
import '../../util/font/font.dart';
import 'package:http/http.dart' as http;

class AdminAddScreen extends StatefulWidget {
  const AdminAddScreen({Key? key}) : super(key: key);

  @override
  State<AdminAddScreen> createState() => _AdminAddScreenState();
}

class _AdminAddScreenState extends State<AdminAddScreen> {

  List _titleL = [
    '이름',
    '이메일',
    '전화번호'
  ];

  List _hintL = [
    '이름을 입력해주세요',
    '이메일을 입력해주세요',
    '전화번호를 입력해주세요'
  ];

  List<TextEditingController> _textConL = [];
  bool emailChecking = false;
  @override
  void initState() {
    _textConL = List.generate(3, (index) => TextEditingController());

    super.initState();
  }


  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(
          '관리자 추가',
          style: f16w900Size(),
        ),
        centerTitle: true,
        backgroundColor: Colors.white,
      ),
      backgroundColor:   Colors.white,
      body: Padding(
        padding: const EdgeInsets.symmetric(horizontal: 20,vertical: 10),
        child: ListView.builder(
            shrinkWrap: true,
            physics: const ClampingScrollPhysics(),
            itemCount: 3,
            itemBuilder: (_, index) {
              return Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  SizedBox(height: 10,),
                  Text(_titleL[index],
                    style: f16w700Size(),
                  ),
                  const SizedBox(height: 10,),
                  TextFormField(
                    controller: _textConL[index],
                    inputFormatters: index==2?
                        [
                          FilteringTextInputFormatter.digitsOnly,
                          MaskTextInputFormatter(mask: '###-####-####'),
                          LengthLimitingTextInputFormatter(13),
                        ]
                        : [
                      LengthLimitingTextInputFormatter(40),

                    ],
                    decoration: InputDecoration(
                      hintText: _hintL[index],
                      hintStyle: hintf14w400Size(),
                      contentPadding: EdgeInsets.symmetric(horizontal: 15,vertical: 16),
                      filled: true,
                      fillColor: Color(0xFFF1F4F7),
                      enabledBorder: UnderlineInputBorder(
                        borderRadius: BorderRadius.circular(4),
                        borderSide: BorderSide.none,
                      ),
                      focusedBorder: UnderlineInputBorder(
                        borderRadius: BorderRadius.circular(4),
                        borderSide: BorderSide.none,
                      ),
                    ),
                  ),
                ],
              );
            }),
      ),
      bottomNavigationBar: GestureDetector(
        onTap: ()async{
          /// 이메일 중복 체크
          await checkEmail();

          if(emailChecking){
            showConfirmTapDialog(context, '관리자명 : ${_textConL[0].text}\n\n'
                '이메일 : ${_textConL[1].text}\n\n'
                '전화번호 : ${formatPhoneNumber(_textConL[2].text)}\n\n 가 맞습니까?',()async{
              await addUserInfo().then((value) {
                Get.back();
                showConfirmTapDialog(context, '관리자가 추가되었습니다', () {
                  Get.back();
                  Get.back();
                });
              });
            });
          }else{
            showOnlyConfirmDialog(context, '이메일이 중복입니다');
          }
        },
        child: Container(
          width: Get.width,
          height: 60,
          color:   const Color(0xff1955EE),
          padding: const EdgeInsets.symmetric(vertical: 16),
          child: Center(child: Text('추가',style: f18w700WhiteSize(),)),
        ),
      ),
    );
  }

  String formatPhoneNumber(String phoneNumber) {
    String cleanedNumber = phoneNumber.replaceAll(RegExp(r'[^0-9]'), '');

    if (cleanedNumber.length == 10) {
      return '${cleanedNumber.substring(0, 3)}-${cleanedNumber.substring(3, 6)}-${cleanedNumber.substring(6)}';
    } else if (cleanedNumber.length == 11) {
      return '${cleanedNumber.substring(0, 3)}-${cleanedNumber.substring(3, 7)}-${cleanedNumber.substring(7)}';
    } else {
      return phoneNumber;
    }
  }

  /// 관리자 추가
  Future<void> addUserInfo() async{
    final us= Get.put(UserState());
    final url = 'http://misnetwork.iptime.org:9000/userInsert';
    final body = ({
      'adminEmail':'${_textConL[1].text}',
      'agency' : '${us.userList[0]['agency']}',
      'agencyNum' : '${us.userList[0]['agencyNumber']}',
      'addressName' : '',
      'addressDetail' : '',
      'adminName' : '${_textConL[0].text}', /// 이름 넣는곳
      'adminPhone': '${_textConL[2].text}',
      'bisNum' : '',
      'mms' : '${us.userList[0]['mms']}',
      'head': 'false',
      'ipcamId' : '',
      'memo' : '',
      'installPrice' : '',
      'userPrice': '',
      'checkIndex': '${us.userList[0]['group']}', /// 그룹
      'contractDay':'',
      'sido': '${us.userList[0]['sido']}',
      'sigungu':'${us.userList[0]['sigungu']}',
      'setupDay' : '',
      'checkinfo': us.userList[0]['mmsType'], /// mms타입이 들어감
      'contractDayRange' : ''
    });
    final response = await http.post(Uri.parse(url), body: body);
    print('?? 응답값${response.body}');
    // List<dynamic> dataList = json.decode(response.body);
    if (response.statusCode != 200) {
      print('에러에러');
      throw Exception('Failed to send email');
    }
  }
  /// 아이디 중복 체크
  Future<void> checkEmail() async {
    final url = 'http://misnetwork.iptime.org:9000/checkEmailSetting?email=${_textConL[1].text}';
    final response = await http.get(Uri.parse(url));
    List<dynamic> dataList = json.decode(response.body);

    if(dataList.isEmpty){
      print('중복없음 가입가능');
      emailChecking = true;
    }else{
      emailChecking = false;
    }
    setState(() {});
    // List<dynamic> dataList = json.decode(response.body);
    if (response.statusCode != 200) {
      print('에러에러');
      throw Exception('Failed to send email');
    }
  }
}
