import 'package:flutter/material.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:get/get.dart';

import '../../provider/user_state.dart';
import '../util/font/font.dart';
import 'admin/admin_setting_screen.dart';
import 'notification/setting_notification_screen.dart';
import 'private/private_change_screen.dart';

class SettingMain extends StatefulWidget {
  const SettingMain({Key? key}) : super(key: key);

  @override
  State<SettingMain> createState() => _SettingMainState();
}

class _SettingMainState extends State<SettingMain> {
  final us = Get.put(UserState());
  static final storage = new FlutterSecureStorage();

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        backgroundColor: Colors.white,
        automaticallyImplyLeading: false,
        actions: [
          Padding(
            padding: const EdgeInsets.only(right: 20),
            child: Text(
              '로그아웃',
              style: f16w700Blue(),
            ),
          )
        ],
      ),
      backgroundColor: Colors.white,
      body: Padding(
        padding: const EdgeInsets.symmetric(horizontal: 20),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [

            /// 개인 정보 변경
            GestureDetector(
              behavior: HitTestBehavior.opaque,
              onTap: () {
                Get.to(() => PrivateChange());
              },
              child: Row(
                children: [
                  Text(
                    '개인정보 변경',
                    style: f18w700Size(),
                  ),
                  const Spacer(),
                  SvgPicture.asset(
                    'assets/icon/rightArrow.svg',
                  ),
                ],
              ),
            ),
            const SizedBox(
              height: 20,
            ),

            /// 알림 설정
            GestureDetector(
              behavior: HitTestBehavior.opaque,
              onTap: () {
                Get.to(() => SettingNotificationScreen());
              },
              child: Row(
                children: [
                  Text(
                    '알림 설정',
                    style: f18w700Size(),
                  ),
                  Spacer(),
                  SvgPicture.asset(
                    'assets/icon/rightArrow.svg',
                  ),
                ],
              ),
            ),
            const SizedBox(
              height: 20,
            ),

            /// 관리자 설정
            GestureDetector(
              behavior: HitTestBehavior.opaque,
              onTap: () {
                Get.to(()=>AdminSetting());
              },
              child: Row(
                children: [
                  Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text(
                        '관리자 설정',
                        style: f18w700Size(),
                      ),
                      Text(
                        '(주관리자만 사용할 수 있습니다)',
                        style: f13w500Grey(),
                      ),
                    ],
                  ),
                  Spacer(),
                  SvgPicture.asset(
                    'assets/icon/rightArrow.svg',
                  ),
                ],
              ),
            ),

            const SizedBox(
              height: 20,
            ),
            Row(
              children: [
                Text(
                  '글자 크기',
                  style: f18w700Size(),
                ),
                Spacer(),
                SizedBox(
                  width: 130,
                  height: 40,
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    crossAxisAlignment: CrossAxisAlignment.center,
                    children: [
                      GestureDetector(
                          onTap: () async {
                            await storage.write(key: "fontSizes", value: '2');
                            us.userFont.value = 2;
                            setState(() {});
                          },
                          child: Text(
                            '소',
                            style: us.userFont.value == 2 ? f14w700 : f14w700BlurGrey,
                          )),
                      GestureDetector(
                          onTap: () async {
                            await storage.write(key: "fontSizes", value: '1');
                            us.userFont.value = 1;
                            setState(() {});
                          },
                          child: Text(
                            '중',
                            style: us.userFont.value == 1 ? f22w700 : f18w700BlurGrey,
                          )),
                      GestureDetector(
                          onTap: () async {
                            await storage.write(key: "fontSizes", value: '0');
                            us.userFont.value = 0;
                            setState(() {});
                          },
                          child: Text(
                            '대',
                            style: us.userFont.value == 0 ? f24w700 : f24w700BlurGrey,
                          )),
                    ],
                  ),
                )
              ],
            ),

            const SizedBox(
              height: 20,
            ),
            GestureDetector(
              behavior: HitTestBehavior.opaque,
              onTap: () {},
              child: Row(
                children: [
                  Text(
                    '버전 정보',
                    style: f18w700Size(),
                  ),
                  Spacer(),
                  Text('v 1.0.0',style: f18w700Size(),)
                ],
              ),
            ),

            // GestureDetector(
            //   behavior: HitTestBehavior.opaque,
            //   onTap: (){
            //     Get.to(()=>PrivateChange());
            //   },
            //     child: Text('개인정보 변경',style: f24w700Size(),)),
            // const SizedBox(height: 35,),
            // Text('알림 설정',style: f24w700Size(),),
            // const SizedBox(height: 35,),
            // GestureDetector(
            //   behavior: HitTestBehavior.opaque,
            //   onTap: (){
            //     Get.to(()=>AdminSetting());
            //   },
            //     child: Text('관리자 설정',style: f20w700Size(),)),
            // Text('(주 관리자만 사용할 수 있습니다)',style: hintf18w400Size(),),
            // const SizedBox(height: 35,),
            // Row(
            //   mainAxisAlignment: MainAxisAlignment.spaceBetween,
            //   children: [
            //     Text('글자 크기',style: f24w700Size(),),
            //     Container(
            //       width: 120,
            //       child: Row(
            //         mainAxisAlignment: MainAxisAlignment.spaceBetween,
            //         children: [
            //           GestureDetector(
            //               onTap:()async{
            //                 await storage.write(key: "fontSizes", value: '2');
            //                 us.userFont.value = 2;
            //                 print('폰트 ?? ${us.userFont.value}');
            //                 setState(() {});
            //               },
            //               child: Text('소', style: us.userFont.value== 2?f20w700:f20w500, )), // textAlign 추가
            //           GestureDetector(
            //             onTap: ()async{
            //               await storage.write(key: "fontSizes", value: '1');
            //               us.userFont.value = 1;
            //               print('폰트 ?? ${us.userFont.value}');
            //               setState(() {});
            //             },
            //               child: Text('중', style: us.userFont.value==1?f22w700:f22w500, )), // textAlign 추가
            //           GestureDetector(
            //             onTap: ()async{
            //               await storage.write(key: "fontSizes", value: '0');
            //               us.userFont.value = 0;
            //               print('폰트 ?? ${us.userFont.value}');
            //               setState(() {});
            //             },
            //               child: Text('대', style: us.userFont.value==0?f24w700:f24w500,)), // textAlign 추가
            //         ],
            //       ),
            //     )
            //   ],
            // ),
            // const SizedBox(height: 35,),
            // Row(
            //   mainAxisAlignment: MainAxisAlignment.spaceBetween,
            //   children: [
            //     Text('버전 정보',style: f24w700Size(),),
            //     Text('v 1.0.0',style: f24w700Size(),)
            //   ],
            // ),
          ],
        ),
      ),
    );
  }
}
